@@ -4,21 +4,41 @@
 (*                                                                     *)
 (*                Daniel de Rauglaudre, INRIA Rocquencourt             *)
 (*                                                                     *)
-(*  Copyright 2001-2010 Institut National de Recherche en Informatique *)
+(*  Copyright 2001-2014 Institut National de Recherche en Informatique *)
 (*  et Automatique.  Distributed only by permission.                   *)
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: ledit.ml,v 1.68 2012-02-08 09:03:40 deraugla Exp $ *)
+(* $Id: ledit.ml,v 1.72 2018/01/24 16:47:00 deraugla Exp $ *)
 
-#load "pa_local.cmo";
-#load "pa_def.cmo";
+#load "pa_macro.cmo";
 #load "pa_fstream.cmo";
+#load "pa_local.cmo";
 
 open Printf;
 
 type encoding = [ Ascii | Iso_8859 | Utf_8 ];
 
+value string_make =
+  IFDEF OCAML_VERSION < OCAML_4_02_0 THEN String.make
+  ELSE Bytes.make END
+;
+
+value string_create =
+  IFDEF OCAML_VERSION < OCAML_4_02_0 THEN String.create
+  ELSE Bytes.create END
+;
+
+value string_length =
+  IFDEF OCAML_VERSION < OCAML_4_02_0 THEN String.length
+  ELSE Bytes.length END
+;
+
+value string_unsafe_set =
+  IFDEF OCAML_VERSION < OCAML_4_02_0 THEN String.unsafe_set
+  ELSE Bytes.unsafe_set END
+;
+
 module A :
   sig
     value encoding : ref encoding;
@@ -119,18 +139,18 @@ module A :
         ;
         value uppercase c =
           match encoding.val with
-          [ Ascii | Iso_8859 -> String.uppercase c
+          [ Ascii | Iso_8859 -> String.uppercase_ascii c
           | Utf_8 ->
               if String.length c = 1 then
-                if Char.code c.[0] < 128 then String.uppercase c else c
+                if Char.code c.[0] < 128 then String.uppercase_ascii c else c
               else c ]
         ;
         value lowercase c =
           match encoding.val with
-          [ Ascii | Iso_8859 -> String.lowercase c
+          [ Ascii | Iso_8859 -> String.lowercase_ascii c
           | Utf_8 ->
               if String.length c = 1 then
-                if Char.code c.[0] < 128 then String.lowercase c else c
+                if Char.code c.[0] < 128 then String.lowercase_ascii c else c
               else c ]
         ;
         value to_string s = s;
@@ -153,12 +173,12 @@ module A :
         ;
         value input ic = get_char (fun () -> input_char ic);
         value read =
-          let buff = " " in
+          let buff = string_make 2 ' ' in
           fun () ->
             get_char
               (fun () ->
                  let len = Unix.read Unix.stdin buff 0 1 in
-                 if len == 0 then raise End_of_file else buff.[0])
+                 if len == 0 then raise End_of_file else Bytes.get buff 0)
         ;
         value parse s = get_char (fun () -> Stream.next s);
         value print c =
@@ -443,7 +463,14 @@ value init_default_commands kb =
      ("\\e[OA", Previous_history);
      ("\\e[OC", Forward_char);
      ("\\e[OD", Backward_char);
-     ("\\e[OH", Beginning_of_line) ::
+     ("\\e[OH", Beginning_of_line);
+     (* gnome-terminal *)
+     ("\\eOH", Beginning_of_line); (* Home *)
+     ("\\eOF", End_of_line); (* End *)
+     (* rxvt *)
+     ("\\e[7~", Beginning_of_line); (* Home *)
+     ("\\e[8~", End_of_line) (* End *)
+     ::
      if meta_as_escape.val then
        [("\\M-b", Backward_word);
         ("\\M-c", Capitalize_word);
@@ -462,11 +489,11 @@ value init_default_commands kb =
 (* Reading the leditrc file *)
 
 value rev_implode l =
-  let s = String.create (List.length l) in
-  loop (String.length s - 1) l where rec loop i =
+  let s = string_create (List.length l) in
+  loop (string_length s - 1) l where rec loop i =
     fun
-    [ [c :: l] -> do { String.unsafe_set s i c; loop (i - 1) l }
-    | [] -> s ]
+    [ [c :: l] -> do { string_unsafe_set s i c; loop (i - 1) l }
+    | [] -> Bytes.to_string s ]
 ;
 
 value rec parse_string rev_cl =
@@ -1637,7 +1664,7 @@ and open_histfile trunc file = do {
     Unix.openfile file
       ([Unix.O_WRONLY; Unix.O_CREAT] @
        (if trunc then [Unix.O_TRUNC] else []))
-      0o666
+      0o600
   in
   let fdo = Unix.out_channel_of_descr fd in
   if not trunc then seek_out fdo (out_channel_length fdo) else ();
